#!/usr/bin/env python3

from pygt1000 import GT1000
from time import sleep

gt1000 = GT1000()
# gt1000.open_ports()
# gt1000.fetch_patch_names()

# sleep(2)
# gt1000.get_fx1()

while True:
    sleep(1)
